@@ -3,7 +3,7 @@ from typing import Tuple, Dict, Any
 import os
 import logging
 
-log = logging.getLogger()
+log = logging.getLogger("archetypes")
 
 
 class Archetype:
@@ -45,7 +45,7 @@ class Archetypes:
                 if f.startswith("."):
                     continue
                 relpath = os.path.relpath(os.path.join(root, f), self.root)
-                for feature_name, feature in self.site.features.items():
+                for feature in self.site.features.ordered():
                     a = feature.try_load_archetype(self, relpath, name)
                     if a is not None:
                         return a
