@@ -1,12 +1,14 @@
+from __future__ import annotations
 import os
 import time
+import locale
 from collections import defaultdict
-from .command import SiteCommand
+from .command import SiteCommand, Fail
 from staticsite.render import File
 from staticsite.utils import timings
 import logging
 
-log = logging.getLogger()
+log = logging.getLogger("build")
 
 
 class Build(SiteCommand):
@@ -24,6 +26,10 @@ class Build(SiteCommand):
 class Builder:
     def __init__(self, site):
         self.site = site
+        if self.site.settings.OUTPUT is None:
+            raise Fail(
+                    "No output directory configured:"
+                    " please use --output or set OUTPUT in settings.py or .staticsite.py")
         self.output_root = os.path.join(site.settings.PROJECT_ROOT, site.settings.OUTPUT)
         self.existing_paths = {}
 
@@ -31,6 +37,13 @@ class Builder:
         """
         Generate output
         """
+        # Set locale for rendering
+        try:
+            lname = self.site.settings.LANGUAGES[0]["locale"]
+            locale.setlocale(locale.LC_ALL, lname)
+        except locale.Error as e:
+            log.warn("Cannot set locale to %s: %s", lname, e)
+
         # Scan the target directory to take note of existing contents
         with timings("Scanned old content in %fs"):
             self.existing_paths = {}
@@ -118,12 +131,14 @@ class Builder:
         counts = defaultdict(int)
 
         # group by type
+        # This is not really needed, and we can render in any order, but this
+        # way we can easily collect rendering timings by page type
         by_type = defaultdict(list)
         for page in pages:
-            by_type[(page.RENDER_PREFERRED_ORDER, page.TYPE)].append(page)
+            by_type[page.TYPE].append(page)
 
         # Render collecting timing statistics
-        for (order, type), pgs in sorted(by_type.items(), key=lambda x: x[0][0]):
+        for type, pgs in sorted(by_type.items(), key=lambda x: x[0][0]):
             start = time.perf_counter()
             for page in pgs:
                 contents = page.render()
