@@ -1,3 +1,4 @@
+from __future__ import annotations
 import shutil
 from .file import File
 
@@ -24,10 +25,10 @@ class RenderedFile(RenderedElement):
         self.src = src
 
     def write(self, dst: File):
-        if dst.stat is not None and (
+        if dst.stat is None or (
                 self.src.stat.st_mtime > dst.stat.st_mtime
                 or self.src.stat.st_size != dst.stat.st_size):
-            shutil.copy2(self.abspath, dst.path)
+            shutil.copy2(self.src.abspath, dst.abspath)
 
     def content(self):
         with open(self.src.abspath, "rb") as fd:
@@ -36,7 +37,10 @@ class RenderedFile(RenderedElement):
 
 class RenderedString(RenderedElement):
     def __init__(self, s):
-        self.buf = s.encode("utf-8")
+        if s is None:
+            self.buf = b"Error in page: see build logs"
+        else:
+            self.buf = s.encode("utf-8")
 
     def write(self, dst: File):
         with open(dst.abspath, "wb") as out:
