@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import Optional, Dict, List, Set, Any, Union, Callable, TYPE_CHECKING
+from typing import Optional, Dict, List, Set, Any, Union, TYPE_CHECKING
 import os
 import datetime
 import pytz
@@ -45,14 +45,7 @@ class Site:
             self.settings.CONTENT = "."
 
         # Repository of metadata descriptions
-        self.metadata: Dict[str, Metadata] = {}
-
-        # Functions called on a page to cleanup metadata on page load
-        self.metadata_on_load_functions: List[Callable[Page], None] = []
-
-        # Functions called on a page to cleanup metadata at the beginning of
-        # the analyze pass
-        self.metadata_on_analyze_functions: List[Callable[Page], None] = []
+        self.metadata = metadata.Registry(self)
 
         # Site pages indexed by site_path
         self.pages: Dict[str, Page] = {}
@@ -105,20 +98,20 @@ class Site:
             self.caches = DisabledCaches()
 
         # Content root for the website
-        self.content_root = os.path.join(self.settings.PROJECT_ROOT, self.settings.CONTENT)
+        self.content_root = os.path.normpath(os.path.join(self.settings.PROJECT_ROOT, self.settings.CONTENT))
 
         # List of content roots scanned for this site
         self.content_roots: List[contents.Dir] = []
 
         # Register well-known metadata
-        self.register_metadata(Metadata("template", inherited=False, doc="""
+        self.register_metadata(metadata.MetadataDefault("template", default="page.html", doc="""
 Template used to render the page. Defaults to `page.html`, although specific
 pages of some features can default to other template names.
 
 Use this similarly to [Jekill's layouts](https://jekyllrb.com/docs/step-by-step/04-layouts/).
 """))
 
-        self.register_metadata(metadata.MetadataDate("date", inherited=False, doc="""
+        self.register_metadata(metadata.MetadataDate("date", doc="""
 Publication date for the page.
 
 A python datetime object, timezone aware. If the date is in the future when
@@ -128,10 +121,33 @@ A python datetime object, timezone aware. If the date is in the future when
 If missing, the modification time of the file is used.
 """))
 
-        self.register_metadata(Metadata("author", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataInherited("author", doc="""
             A string with the name of the author for this page.
+
+            SITE_AUTHOR is used as a default if found in settings.
+
+            If not found, it defaults to the current user's name.
         """))
-        self.register_metadata(Metadata("title", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataTemplateInherited("template_copyright", template_for="copyright", doc="""
+If set instead of `copyright`, it is a jinja2 template used to generate the
+copyright information.
+
+The template context will have `page` available, with the current page. The
+result of the template will not be further escaped, so you can use HTML markup
+in it.
+
+If missing, defaults to `"© {{page.meta.date.year}} {{page.meta.author}}"`
+"""))
+        self.register_metadata(metadata.MetadataInherited("copyright", doc="""
+            A string with the copyright information for this page.
+        """))
+        self.register_metadata(metadata.MetadataTemplateInherited("template_title", template_for="title", doc="""
+If set instead of `title`, it is a jinja2 template used to generate the title.
+The template context will have `page` available, with the current page. The
+result of the template will not be further escaped, so you can use HTML markup
+in it.
+"""))
+        self.register_metadata(metadata.MetadataInherited("title", doc="""
 The page title.
 
 If omitted:
@@ -143,32 +159,26 @@ If omitted:
    inherited.
  * if still no title can be found, the site name is used as a default.
 """))
-        self.register_metadata(Metadata("template_title", inherited=True, template_for="title", doc="""
-If set instead of title, it is a jinja2 template used to generate the title.
-The template context will have `page` available, with the current page. The
-result of the template will not be further escaped, so you can use HTML markup
-in it.
-"""))
-        self.register_metadata(Metadata("description", inherited=True, doc="""
-The page description. If omitted, the page will have no description.
-"""))
-        self.register_metadata(Metadata("template_description", inherited=True, template_for="description", doc="""
-If set instead of description, it is a jinja2 template used to generate the
+        self.register_metadata(metadata.MetadataTemplateInherited("template_description", template_for="description", doc="""
+If set instead of `description`, it is a jinja2 template used to generate the
 description. The template context will have `page` available, with the current
 page. The result of the template will not be further escaped, so you can use
 HTML markup in it.
+"""))
+        self.register_metadata(metadata.MetadataInherited("description", doc="""
+The page description. If omitted, the page will have no description.
 """))
 
-        self.register_metadata(Metadata("site_name", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataInherited("site_name", doc="""
 Name of the site. If missing, it defaults to the title of the toplevel index
 page. If missing, it defaults to the name of the content directory.
 """))
 
-        self.register_metadata(Metadata("site_url", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataInherited("site_url", doc="""
 Base URL for the site, used to generate an absolute URL to the page.
 """))
 
-        self.register_metadata(Metadata("site_path", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataSitePath("site_path", doc="""
 Where a content directory appears in the site.
 
 By default, is is the `site_path` of the parent directory, plus the directory
@@ -178,7 +188,7 @@ If you are publishing the site at `/prefix` instead of the root of the domain,
 override this with `/prefix` in the content root.
 """))
 
-        self.register_metadata(Metadata("build_path", inherited=False, doc="""
+        self.register_metadata(Metadata("build_path", doc="""
 Relative path in the build directory for the file that will be written
 when this page gets rendered. For example, `blog/2016/example.md`
 generates `blog/2016/example/index.html`.
@@ -187,7 +197,7 @@ If found in pages front matter, it is ignored, and is always computed at page
 load time.
 """))
 
-        self.register_metadata(Metadata("asset", inherited=True, doc="""
+        self.register_metadata(metadata.MetadataInherited("asset", doc="""
 If set to True for a file (for example, by a `file:` pattern in a directory
 index), the file is loaded as a static asset, regardless of whether a feature
 would load it.
@@ -196,13 +206,13 @@ If set to True in a directory index, the directory and all its subdirectories
 are loaded as static assets, without the interventions of features.
 """))
 
-        self.register_metadata(Metadata("aliases", inherited=False, structure=True, doc="""
+        self.register_metadata(metadata.MetadataInherited("aliases", doc="""
 Relative paths in the destination directory where the page should also show up.
 [Like in Hugo](https://gohugo.io/extras/aliases/), this can be used to maintain
 existing links when moving a page to a different location.
 """))
 
-        self.register_metadata(metadata.MetadataIndexed("indexed", inherited=False, doc="""
+        self.register_metadata(metadata.MetadataIndexed("indexed", doc="""
 If true, the page appears in [directory indices](dir.md) and in
 [page filter results](page_filter.md).
 
@@ -210,7 +220,7 @@ It defaults to true at least for [Markdown](markdown.md),
 [reStructuredText](rst.rst), and [data](data.md) pages.
 """))
 
-        self.register_metadata(metadata.MetadataDraft("draft", inherited=False, doc="""
+        self.register_metadata(metadata.MetadataDraft("draft", doc="""
 If true, the page is still a draft and will not appear in the destination site,
 unless draft mode is enabled.
 
@@ -229,16 +239,7 @@ It defaults to false, or true if `page.meta.date` is in the future.
         """
         if self.stage_content_directory_scanned:
             log.warn("register_metadata called after content directory has been scanned")
-        metadata.site = self
-        self.metadata[metadata.name] = metadata
-
-        on_load = getattr(metadata, "on_load", None)
-        if not getattr(on_load, "skip", False):
-            self.metadata_on_load_functions.append(on_load)
-
-        on_analyze = getattr(metadata, "on_analyze", None)
-        if not getattr(on_analyze, "skip", False):
-            self.metadata_on_analyze_functions.append(on_analyze)
+        self.metadata.add(metadata)
 
     @lazy
     def archetypes(self) -> "archetypes.Archetypes":
@@ -276,17 +277,28 @@ It defaults to false, or true if `page.meta.date` is in the future.
         """
         Build directory metadata based on site settings
         """
-        meta = {}
+        meta = {
+            "template_copyright": "© {{page.meta.date.year}} {{page.meta.author}}",
+        }
         if self.settings.SITE_URL:
             meta["site_url"] = self.settings.SITE_URL
         if self.settings.SITE_ROOT:
-            meta["site_path"] = self.settings.SITE_ROOT.lstrip("/")
+            meta["site_path"] = os.path.normpath(os.path.join("/", self.settings.SITE_ROOT))
         else:
-            meta["site_path"] = ""
+            meta["site_path"] = "/"
         if self.settings.SITE_NAME:
             meta["site_name"] = self.settings.SITE_NAME
         if self.settings.SITE_AUTHOR:
             meta["author"] = self.settings.SITE_AUTHOR
+        else:
+            import getpass
+            import pwd
+            user = getpass.getuser()
+            pw = pwd.getpwnam(user)
+            meta["author"] = pw.pw_gecos.split(",")[0]
+
+        log.debug("Initial settings: %r", meta)
+
         return meta
 
     def scan_content(self):
@@ -301,7 +313,7 @@ It defaults to false, or true if `page.meta.date` is in the future.
             log.info("%s: content tree does not exist", self.content_root)
             return
 
-        src = File("", os.path.abspath(self.content_root), os.stat(self.content_root))
+        src = File.with_stat("", os.path.abspath(self.content_root))
         self.scan_tree(src, meta=self._settings_to_meta())
         self.theme.scan_assets()
         self.stage_content_directory_scanned = True
@@ -364,13 +376,21 @@ It defaults to false, or true if `page.meta.date` is in the future.
             log.info("%s: page is still a draft: skipping", page)
             return
 
+        # Mount page by site path
         site_path = page.meta["site_path"]
         old = self.pages.get(site_path)
         if old is not None:
-            if old.TYPE != "asset" or page.TYPE != "asset":
+            if old.TYPE == "asset" and page.TYPE == "asset":
+                pass
+            # elif old.TYPE == "dir" and page.TYPE not in ("dir", "asset"):
+            #     pass
+            else:
                 log.warn("%s: replacing page %s", page, old)
         self.pages[site_path] = page
-        if page.src is not None:
+
+        # Mount page by src.relpath
+        # Skip pages derived from other pages, or they would overwrite them
+        if page.src is not None and not page.created_from:
             self.pages_by_src_relpath[page.src.relpath] = page
 
         # Also group pages by tracked metadata
@@ -388,9 +408,10 @@ It defaults to false, or true if `page.meta.date` is in the future.
             log.warn("analyze called before loading site contents")
 
         # Run metadata on_analyze functions
-        for f in self.metadata_on_analyze_functions:
-            for page in self.pages.values():
-                f(page)
+        for page in self.pages.values():
+            if page.meta.get("asset", False):
+                continue
+            self.metadata.on_analyze(page)
 
         # Add missing pages_by_metadata entries in case no matching page were
         # found for some of them
@@ -413,9 +434,7 @@ It defaults to false, or true if `page.meta.date` is in the future.
         return slugify(text)
 
     def localized_timestamp(self, ts):
-        return pytz.utc.localize(
-                        datetime.datetime.utcfromtimestamp(ts)
-                    ).astimezone(self.timezone)
+        return datetime.datetime.fromtimestamp(ts, tz=pytz.utc).astimezone(self.timezone)
 
     # TODO: support partial dates?
     re_isodate = re.compile(r"^(\d{4}-\d{2}-\d{2}[ T]\d{2}:\d{2}:\d{2})(Z|\+\d{2}.*)$")
