@@ -9,7 +9,6 @@ from collections import defaultdict
 from .page import Page, PageNotFoundError
 from .utils import front_matter, arrange
 from .utils.typing import Meta
-from .page_filter import PageFilter
 from .metadata import Metadata
 from .file import File
 from . import toposort
@@ -96,8 +95,8 @@ class Loader:
         if not os.path.isfile(pathname):
             config = {}
         else:
-            with open(pathname, "rb") as fd:
-                fmt, config = front_matter.read_partial(fd)
+            with open(pathname, "rt") as fd:
+                fmt, config = front_matter.read_whole(fd)
 
         # Normalize 'extends' to a list of strings
         extends = config.get("extends")
@@ -115,6 +114,48 @@ class Loader:
         return config
 
 
+class Jinja2TemplateLoader(jinja2.loaders.BaseLoader):
+    re_content = re.compile(r"^content:(.+)")
+
+    def __init__(self, theme):
+        self.loader_content = jinja2.FileSystemLoader(theme.site.content_root)
+        self.loader_theme = jinja2.FileSystemLoader(theme.template_lookup_paths)
+
+    def get_loader(self, template):
+        mo = self.re_content.match(template)
+        if mo:
+            return self.loader_content, mo.group(1)
+        else:
+            return self.loader_theme, template
+
+    def get_source(self, environment, template):
+        loader, name = self.get_loader(template)
+        try:
+            return loader.get_source(environment, name)
+        except jinja2.TemplateNotFound:
+            # re-raise the exception with the correct filename here.
+            # (the one that includes the prefix)
+            raise jinja2.TemplateNotFound(template)
+
+    @jinja2.utils.internalcode
+    def load(self, environment, name, globals=None):
+        loader, local_name = self.get_loader(name)
+        try:
+            return loader.load(environment, local_name, globals)
+        except jinja2.TemplateNotFound:
+            # re-raise the exception with the correct filename here.
+            # (the one that includes the prefix)
+            raise jinja2.TemplateNotFound(name)
+
+    def list_templates(self):
+        result = []
+        for template in self.loader_theme.list_templates():
+            result.append(template)
+        for template in self.loader_content.list_templates():
+            result.append("content:" + template)
+        return result
+
+
 class Theme:
     def __init__(self, site, name, configs: List[Meta]):
         # Site object
@@ -134,7 +175,7 @@ class Theme:
         self.metadata_templates: Optional[List[Metadata]] = None
 
         # Compute template lookup paths
-        self.template_lookup_paths: List[str] = [self.site.content_root]
+        self.template_lookup_paths: List[str] = []
         for config in reversed(self.configs):
             self.template_lookup_paths.append(config["root"])
         log.info("%s: template lookup paths: %r", self.name, self.template_lookup_paths)
@@ -161,6 +202,13 @@ class Theme:
                 self.theme_static_dirs.append(theme_static)
         log.info("%s: theme static directories: %r", self.name, self.theme_static_dirs)
 
+        # Merge theme metadata
+        meta_keys = frozenset(("image_sizes",))
+        self.meta: Meta = {}
+        for config in self.configs:
+            for key in config.keys() & meta_keys:
+                self.meta[key] = config[key]
+
     @classmethod
     def create(cls, site, name: str, search_paths: Sequence[str] = None) -> "Theme":
         """
@@ -194,7 +242,6 @@ class Theme:
         self.site.stage_features_constructed = True
 
         # Jinja2 template engine
-        from jinja2 import FileSystemLoader
         if self.site.settings.JINJA2_SANDBOXED:
             from jinja2.sandbox import ImmutableSandboxedEnvironment
             env_cls = ImmutableSandboxedEnvironment
@@ -203,7 +250,7 @@ class Theme:
             env_cls = Environment
 
         self.jinja2 = env_cls(
-            loader=FileSystemLoader(self.template_lookup_paths),
+            loader=Jinja2TemplateLoader(self),
             autoescape=True,
         )
 
@@ -222,6 +269,7 @@ class Theme:
             url_for=self.jinja2_url_for,
             page_for=self.jinja2_page_for,
             site_pages=self.jinja2_site_pages,
+            img_for=self.jinja2_img_for,
             now=self.site.generation_time,
             next_month=(
                 self.site.generation_time.replace(day=1) + datetime.timedelta(days=40)).replace(
@@ -255,14 +303,14 @@ class Theme:
             fmeta["site_path"] = os.path.join(site_path, name)
             # TODO: make this a child of the previously scanned static
             self.site.scan_tree(
-                src=File(name, root, os.stat(root)),
+                src=File.with_stat(name, root),
                 meta=fmeta,
             )
 
         # Load assets from theme directories
         for path in self.theme_static_dirs:
             self.site.scan_tree(
-                src=File("", os.path.abspath(path), os.stat(path)),
+                src=File.with_stat("", os.path.abspath(path)),
                 meta=meta,
             )
 
@@ -272,7 +320,7 @@ class Theme:
         template strings
         """
         if self.metadata_templates is None:
-            self.metadata_templates = [m for m in self.site.metadata.values() if m.template_for is not None]
+            self.metadata_templates = [m for m in self.site.metadata.values() if m.type == "jinja2"]
         for metadata in self.metadata_templates:
             val = meta.get(metadata.name)
             if val is None:
@@ -280,25 +328,6 @@ class Theme:
             if isinstance(val, str):
                 meta[metadata.name] = self.jinja2.from_string(val)
 
-    def render_metadata_templates(self, page: Page):
-        """
-        Render all the elements of the given metadata that are jinja2
-        template strings
-        """
-        if self.metadata_templates is None:
-            self.metadata_templates = [m for m in self.site.metadata.values() if m.template_for is not None]
-
-        for metadata in self.metadata_templates:
-            src = page.meta.get(metadata.name)
-            if src is None:
-                continue
-            if metadata.template_for in page.meta:
-                continue
-            if isinstance(src, str):
-                src = self.jinja2.from_string(src)
-                page.meta[metadata.name] = src
-            page.meta[metadata.template_for] = jinja2.Markup(page.render_template(src))
-
     def jinja2_basename(self, val: str) -> str:
         return os.path.basename(val)
 
@@ -352,7 +381,7 @@ class Theme:
         try:
             return cur_page.resolve_path(arg)
         except PageNotFoundError as e:
-            log.warn("%s: %s", context.name, e)
+            log.warn("%s:%s: %s", cur_page, context.name, e)
             return ""
 
     @jinja2.contextfunction
@@ -368,11 +397,36 @@ class Theme:
         try:
             return cur_page.url_for(arg, absolute=absolute)
         except PageNotFoundError as e:
-            log.warn("%s: %s", context.name, e)
+            log.warn("%s:%s: %s", cur_page, context.name, e)
             return ""
 
     @jinja2.contextfunction
-    def jinja2_site_pages(
-            self, context, path: str = None, limit: Optional[int] = None, sort: str = "-date", **kw) -> List[Page]:
-        page_filter = PageFilter(self.site, path=path, limit=limit, sort=sort, **kw)
-        return page_filter.filter(self.site.pages.values())
+    def jinja2_site_pages(self, context, **kw) -> List[Page]:
+        cur_page = context.get("page")
+        if cur_page is None:
+            log.warn("%s+%s: site_pages: current page is not defined", cur_page, context.name)
+            return []
+
+        return cur_page.find_pages(**kw)
+
+    @jinja2.contextfunction
+    def jinja2_img_for(
+            self, context,
+            path: Union[str, Page],
+            type: Optional[str] = None,
+            absolute: bool = False,
+            **attrs) -> List[Page]:
+        cur_page = context.get("page")
+        if cur_page is None:
+            log.warn("%s+%s: img(%s): current page is not defined", cur_page, context.name, path)
+            return ""
+
+        res_attrs = cur_page.get_img_attributes(path, type=type, absolute=absolute)
+        res_attrs.update(attrs)
+
+        escape = jinja2.escape
+        res = ["<img"]
+        for k, v in res_attrs.items():
+            res.append(f" {escape(k)}='{escape(v)}'")
+        res.append("></img>")
+        return jinja2.Markup("".join(res))
