@@ -17,8 +17,8 @@ class TestTags(TestCase):
             site.load()
             site.analyze()
 
-            tags = site.pages["taxonomies/tags"]
-            page1 = site.pages["page1"]
+            tags = site.pages["/taxonomies/tags"]
+            page1 = site.pages["/page1"]
 
             self.assertEqual(tags.categories["a"].meta["pages"], [page1])
             self.assertEqual(tags.categories["b"].meta["pages"], [page1])
@@ -40,9 +40,9 @@ tags: [cat]
             site.load()
             site.analyze()
 
-            tags = site.pages["taxonomies/tags"]
+            tags = site.pages["/taxonomies/tags"]
             cat = tags.categories["cat"]
-            page = site.pages["page"]
+            page = site.pages["/page"]
 
             self.assertEqual(cat.meta["pages"], [page])
             self.assertEqual(page.meta["tags"], [cat])
